@@ -57,14 +57,19 @@ message cons. If nil, the response will be a simple vector."
     (with-current-buffer (get-buffer-create (llm-fake-output-to-buffer provider))
       (goto-char (point-max))
       (insert "\nCall to llm-chat\n"  (llm-chat-prompt-to-text prompt) "\n")))
-  (if (llm-fake-chat-action-func provider)
-      (let* ((f (llm-fake-chat-action-func provider))
-             (result (funcall f)))
-        (pcase (type-of result)
-                ('string result)
-                ('cons (signal (car result) (cdr result)))
-                (_ (error "Incorrect type found in `chat-action-func': %s" (type-of result)))))
-    "Sample response from `llm-chat-async'"))
+  (let ((result
+         (if (llm-fake-chat-action-func provider)
+             (let* ((f (llm-fake-chat-action-func provider))
+                    (result (funcall f)))
+               (pcase (type-of result)
+                 ('string result)
+                 ('cons (signal (car result) (cdr result)))
+                 (_ (error "Incorrect type found in `chat-action-func': %s" (type-of result)))))
+           "Sample response from `llm-chat-async'")))
+    (setf (llm-chat-prompt-interactions prompt)
+          (append (llm-chat-prompt-interactions prompt)
+                  (list (make-llm-chat-prompt-interaction :role 'assistant :content result))))
+    result))
 
 (cl-defmethod llm-chat-streaming ((provider llm-fake) prompt partial-callback response-callback _error-callback)
   (when (llm-fake-output-to-buffer provider)
@@ -85,6 +90,9 @@ message cons. If nil, the response will be a simple vector."
               (funcall partial-callback accum)
               (sleep-for 0 100))
             (string-split text))
+      (setf (llm-chat-prompt-interactions prompt)
+            (append (llm-chat-prompt-interactions prompt)
+                    (list (make-llm-chat-prompt-interaction :role 'assistant :content text))))
       (funcall response-callback text))))
 
 (cl-defmethod llm-embedding ((provider llm-fake) string)
